import 'dart:convert';
import 'dart:io';

import 'package:device_info_plus/device_info_plus.dart';

Future<KntDeviceInfo> fetchDeviceInfo() async {
  final deviceInfo = DeviceInfoPlugin();
  if (Platform.isAndroid) {
    final androidInfo = await deviceInfo.androidInfo;
    return KntDeviceInfo(
      id: androidInfo.androidId,
      model: androidInfo.model,
      version: 'Android ${androidInfo.version.release}',
      name: androidInfo.product,
    );
  }
  if (Platform.isIOS) {
    final iosInfo = await deviceInfo.iosInfo;
    return KntDeviceInfo(
      id: iosInfo.identifierForVendor,
      model: iosInfo.model,
      version: '${iosInfo.systemName} ${iosInfo.systemVersion}',
      name: iosInfo.utsname.machine,
    );
  }
  return const KntDeviceInfo();
}

class KntDeviceInfo {
  final String? id;
  final String? model;
  final String? version;
  final String? name;

  const KntDeviceInfo({
    this.id,
    this.model,
    this.version,
    this.name,
  });

  Map<String, dynamic> toMap() => <String, dynamic>{
        'id': id,
        'model': model,
        'version': version,
        'name': name,
      };

  String get toJsonString => jsonEncode(toMap());
}
